/*
 * Copyright 1998 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */

/* @test
   @bug 4143651
   @summary Test if I/O methods will check if the stream
            has been closed.
*/



import java.io.*;

public class ReadAfterClose {
    static void testRead(InputStream in) throws Exception {
        in.close();
        byte[] buf = new byte[2];

        try {
            in.read(buf, 0, 1);
            throw new Exception("Should not allow read on a closed stream");
        } catch (IOException e) {
        }

        try {
            in.read(buf, 0, 0);
            throw new Exception("Should not allow read on a closed stream");
        } catch (IOException e) {
        }
    }

    public static void main(String argv[]) throws Exception {
        BufferedInputStream bis = new BufferedInputStream
            (new ByteArrayInputStream(new byte[32]));
        testRead(bis);
    }
}
