/*
 * Copyright 1998 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */

/* @test
   @bug 4109635
   @summary For an inner class, the access bits must come from the
            InnerClasses attribute, not from the class block's access.
            Note however that the VM should not rely on these access
            flags from the attribute!
 */
import java.lang.reflect.Modifier;
public class ForInnerClass {
    private class Inner {
    }

    protected class Protected {
    }

    public static void main(String[] args) throws Exception {
        /* We are not testing for the ACC_SUPER bug, so strip we strip
         * synchorized. */

        int m = 0;

        m = Inner.class.getModifiers() & (~Modifier.SYNCHRONIZED);
        if (m != Modifier.PRIVATE)
            throw new Exception("Access bits for innerclass not from " +
                                "InnerClasses attribute");

        m = Protected.class.getModifiers() & (~Modifier.SYNCHRONIZED);
        if (m != Modifier.PROTECTED)
            throw new Exception("Protected inner class wronged modifiers");
    }
}
