/*
 * Copyright 2003 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 *
 */

package sun.jvm.hotspot.memory;

import java.io.*;
import java.util.*;
import sun.jvm.hotspot.debugger.*;
import sun.jvm.hotspot.runtime.*;
import sun.jvm.hotspot.types.*;

public class ConcurrentMarkSweepGeneration extends CardGeneration {
  private static AddressField cmsSpaceField;

  public ConcurrentMarkSweepGeneration(Address addr) {
    super(addr);
  }

  static {
    VM.registerVMInitializedObserver(new Observer() {
        public void update(Observable o, Object data) {
          initialize(VM.getVM().getTypeDataBase());
        }
      });
  }

  private static synchronized void initialize(TypeDataBase db) {
    Type type = db.lookupType("ConcurrentMarkSweepGeneration");
    cmsSpaceField = type.getAddressField("_cmsSpace");
  }

  // Accessing space
  public CompactibleFreeListSpace cmsSpace() {
    return (CompactibleFreeListSpace) VMObjectFactory.newObject(
                                 CompactibleFreeListSpace.class,
                                 cmsSpaceField.getValue(addr));
  }

  public long capacity()                { return cmsSpace().capacity(); }
  public long used()                    { return cmsSpace().used(); }
  public long free()                    { return cmsSpace().free(); }
  public long contiguousAvailable()     { throw new RuntimeException("not yet implemented"); }
  public boolean contains(Address p)    { return cmsSpace().contains(p); }
  public void spaceIterate(SpaceClosure blk, boolean usedOnly) {
     blk.doSpace(cmsSpace());
  }

  public Generation.Name kind() {
    return Generation.Name.CONCURRENT_MARK_SWEEP;
  }

  public String name() {
    return "concurrent mark-sweep generation";
  }

  public void printOn(PrintStream tty) {
    tty.println(name());
    cmsSpace().printOn(tty);
  }
}
