/*
 * Copyright 2007 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */

package com.sun.javatest.diff;

import java.io.IOException;
import java.util.Comparator;

import com.sun.javatest.TestResult;
import java.util.ArrayList;
import java.util.List;


/**
 * Interface for generating reports.
 */
public abstract class Reporter {

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public Comparator<TestResult> getComparator() {
        return comparator;
    }

    public void setComparator(Comparator<TestResult> c) {
        this.comparator = c;
    }
    
    public void setReaders(List<DiffReader> readers) {
        this.readers = readers;
    }
    
    public void setTestCounts(List<int[]> testCounts) {
        this.testCounts = testCounts;
    }
    
    public int getDiffCount() {
        return diffs;
    }
    
    abstract void write(MultiMap<String,TestResult> table) throws IOException;
    
    protected List<DiffReader> readers;
    protected List<int[]> testCounts = new ArrayList<int[]>();
    protected Comparator<TestResult> comparator;
    protected String title;
    protected int diffs;
}
